// This is a generated file, modify: generate/templates/class_content.cc.
#include <nan.h>
#include <string.h>
#include <chrono>
#include <thread>

extern "C" {
#include <git2.h>
}

#include "../include/functions/copy.h"
#include "../include/macros.h"
#include "../include/rebase.h"

#include "../include/rebase.h"
#include "../include/signature.h"
#include "../include/oid.h"
#include "../include/rebase_options.h"
#include "../include/repository.h"
#include "../include/annotated_commit.h"
#include "../include/rebase_operation.h"

#include <iostream>

using namespace std;
using namespace v8;
using namespace node;

GitRebase::GitRebase(git_rebase *raw, bool selfFreeing)
{
  this->raw = raw;
  this->selfFreeing = selfFreeing;
}

GitRebase::~GitRebase()
{
  if (this->selfFreeing) {
    git_rebase_free(this->raw);
    this->raw = NULL;
  }

  // this will cause an error if you have a non-self-freeing object that also needs
  // to save values. Since the object that will eventually free the object has no
  // way of knowing to free these values.

}

void GitRebase::InitializeComponent(Handle<v8::Object> target)
{
  NanScope();

  Local<FunctionTemplate> tpl = NanNew<FunctionTemplate>(JSNewFunction);

  tpl->InstanceTemplate()->SetInternalFieldCount(1);
  tpl->SetClassName(NanNew<String>("Rebase"));

  NODE_SET_PROTOTYPE_METHOD(tpl, "abort", Abort);
  NODE_SET_PROTOTYPE_METHOD(tpl, "commit", Commit);
  NODE_SET_PROTOTYPE_METHOD(tpl, "finish", Finish);
  NODE_SET_PROTOTYPE_METHOD(tpl, "free", Free);
  NODE_SET_METHOD(tpl, "init", Init);
  NODE_SET_METHOD(tpl, "initOptions", InitOptions);
  NODE_SET_METHOD(tpl, "open", Open);
  NODE_SET_PROTOTYPE_METHOD(tpl, "operationByIndex", OperationByindex);
  NODE_SET_PROTOTYPE_METHOD(tpl, "operationCurrent", OperationCurrent);
  NODE_SET_PROTOTYPE_METHOD(tpl, "operationEntrycount", OperationEntrycount);

  Local<Function> _constructor_template = tpl->GetFunction();
  NanAssignPersistent(constructor_template, _constructor_template);
  target->Set(NanNew<String>("Rebase"), _constructor_template);
}

NAN_METHOD(GitRebase::JSNewFunction)
{
  NanScope();

  if (args.Length() == 0 || !args[0]->IsExternal()) {
    return NanThrowError("A new GitRebase cannot be instantiated.");
  }

  GitRebase* object = new GitRebase(static_cast<git_rebase *>(Handle<External>::Cast(args[0])->Value()), args[1]->BooleanValue());
  object->Wrap(args.This());

  NanReturnValue(args.This());
}

Handle<v8::Value> GitRebase::New(void *raw, bool selfFreeing)
{
  NanEscapableScope();
  Handle<v8::Value> argv[2] = { NanNew<External>((void *)raw), NanNew<Boolean>(selfFreeing) };
  return NanEscapeScope(NanNew<Function>(GitRebase::constructor_template)->NewInstance(2, argv));
}

git_rebase *GitRebase::GetValue()
{
  return this->raw;
}

git_rebase **GitRebase::GetRefValue()
{
  return this->raw == NULL ? NULL : &this->raw;
}

void GitRebase::ClearValue()
{
  this->raw = NULL;
}


/*
   * @param Signature signature
     * @return Number  result    */
NAN_METHOD(GitRebase::Abort)
{
  NanEscapableScope();

  if (args.Length() == 0 || !args[0]->IsObject()) {
    return NanThrowError("Signature signature is required.");
  }

// start convert_from_v8 block
  const git_signature * from_signature;
  from_signature = ObjectWrap::Unwrap<GitSignature>(args[0]->ToObject())->GetValue();
// end convert_from_v8 block
  int result = git_rebase_abort(
                 ObjectWrap::Unwrap<GitRebase>(args.This())->GetValue()
                 ,from_signature
               );


  Handle<v8::Value> to;
// start convert_to_v8 block
  to = NanNew<Number>( result);
  // end convert_to_v8 block
  NodeGitPsueodoNanReturnEscapingValue(to);
}

/*
 * @param Oid id
     * @param Signature author
   * @param Signature committer
   * @param String message_encoding
   * @param String message
     * @return Number  result    */
NAN_METHOD(GitRebase::Commit)
{
  NanEscapableScope();

  if (args.Length() == 0
      || (!args[0]->IsObject() && !args[0]->IsString())) {
    return NanThrowError("Oid id is required.");
  }
  if (args.Length() == 1 || !args[1]->IsObject()) {
    return NanThrowError("Signature author is required.");
  }

  if (args.Length() == 2 || !args[2]->IsObject()) {
    return NanThrowError("Signature committer is required.");
  }

  if (args.Length() == 3 || !args[3]->IsString()) {
    return NanThrowError("String message_encoding is required.");
  }

  if (args.Length() == 4 || !args[4]->IsString()) {
    return NanThrowError("String message is required.");
  }

// start convert_from_v8 block
  git_oid * from_id;
  if (args[0]->IsString()) {
    // Try and parse in a string to a git_oid
    String::Utf8Value oidString(args[0]->ToString());
    git_oid *oidOut = (git_oid *)malloc(sizeof(git_oid));

    if (git_oid_fromstr(oidOut, (const char *) strdup(*oidString)) != GIT_OK) {
      free(oidOut);

      if (giterr_last()) {
        return NanThrowError(giterr_last()->message);
      } else {
        return NanThrowError("Unknown Error");
      }
    }

    from_id = oidOut;
  } else {
    from_id = ObjectWrap::Unwrap<GitOid>(args[0]->ToObject())->GetValue();
  }
// end convert_from_v8 block
// start convert_from_v8 block
  const git_signature * from_author;
  from_author = ObjectWrap::Unwrap<GitSignature>(args[1]->ToObject())->GetValue();
// end convert_from_v8 block
// start convert_from_v8 block
  const git_signature * from_committer;
  from_committer = ObjectWrap::Unwrap<GitSignature>(args[2]->ToObject())->GetValue();
// end convert_from_v8 block
// start convert_from_v8 block
  const char * from_message_encoding;

  String::Utf8Value message_encoding(args[3]->ToString());
  from_message_encoding = (const char *) strdup(*message_encoding);
// end convert_from_v8 block
// start convert_from_v8 block
  const char * from_message;

  String::Utf8Value message(args[4]->ToString());
  from_message = (const char *) strdup(*message);
// end convert_from_v8 block
  int result = git_rebase_commit(
                 from_id
                 ,ObjectWrap::Unwrap<GitRebase>(args.This())->GetValue()
                 ,from_author
                 ,from_committer
                 ,from_message_encoding
                 ,from_message
               );

  if (args[0]->IsString()) {
    free((void *)from_id);
  }

  Handle<v8::Value> to;
// start convert_to_v8 block
  to = NanNew<Number>( result);
  // end convert_to_v8 block
  NodeGitPsueodoNanReturnEscapingValue(to);
}

/*
   * @param Signature signature
   * @param RebaseOptions opts
     * @return Number  result    */
NAN_METHOD(GitRebase::Finish)
{
  NanEscapableScope();

  if (args.Length() == 0 || !args[0]->IsObject()) {
    return NanThrowError("Signature signature is required.");
  }

  if (args.Length() == 1 || !args[1]->IsObject()) {
    return NanThrowError("RebaseOptions opts is required.");
  }

// start convert_from_v8 block
  const git_signature * from_signature;
  from_signature = ObjectWrap::Unwrap<GitSignature>(args[0]->ToObject())->GetValue();
// end convert_from_v8 block
// start convert_from_v8 block
  const git_rebase_options * from_opts;
  from_opts = ObjectWrap::Unwrap<GitRebaseOptions>(args[1]->ToObject())->GetValue();
// end convert_from_v8 block
  int result = git_rebase_finish(
                 ObjectWrap::Unwrap<GitRebase>(args.This())->GetValue()
                 ,from_signature
                 ,from_opts
               );


  Handle<v8::Value> to;
// start convert_to_v8 block
  to = NanNew<Number>( result);
  // end convert_to_v8 block
  NodeGitPsueodoNanReturnEscapingValue(to);
}

/*
     */
NAN_METHOD(GitRebase::Free)
{
  NanEscapableScope();

  if (ObjectWrap::Unwrap<GitRebase>(args.This())->GetValue() != NULL) {
    git_rebase_free(
      ObjectWrap::Unwrap<GitRebase>(args.This())->GetValue()
    );

    ObjectWrap::Unwrap<GitRebase>(args.This())->ClearValue();
  }
  NanReturnUndefined();
}

/*
  * @param Repository repo
   * @param AnnotatedCommit branch
   * @param AnnotatedCommit upstream
   * @param AnnotatedCommit onto
   * @param Signature signature
   * @param RebaseOptions opts
    * @param Rebase callback
   */
NAN_METHOD(GitRebase::Init)
{
  NanScope();

  if (args.Length() == 0 || !args[0]->IsObject()) {
    return NanThrowError("Repository repo is required.");
  }

  if (args.Length() == 1 || !args[1]->IsObject()) {
    return NanThrowError("AnnotatedCommit branch is required.");
  }

  if (args.Length() == 2 || !args[2]->IsObject()) {
    return NanThrowError("AnnotatedCommit upstream is required.");
  }

  if (args.Length() == 3 || !args[3]->IsObject()) {
    return NanThrowError("AnnotatedCommit onto is required.");
  }

  if (args.Length() == 4 || !args[4]->IsObject()) {
    return NanThrowError("Signature signature is required.");
  }

  if (args.Length() == 5 || !args[5]->IsObject()) {
    return NanThrowError("RebaseOptions opts is required.");
  }

  if (args.Length() == 6 || !args[6]->IsFunction()) {
    return NanThrowError("Callback is required and must be a Function.");
  }

  InitBaton* baton = new InitBaton;

  baton->error_code = GIT_OK;
  baton->error = NULL;

// start convert_from_v8 block
  git_repository * from_repo;
  from_repo = ObjectWrap::Unwrap<GitRepository>(args[0]->ToObject())->GetValue();
// end convert_from_v8 block
  baton->repo = from_repo;
// start convert_from_v8 block
  const git_annotated_commit * from_branch;
  from_branch = ObjectWrap::Unwrap<GitAnnotatedCommit>(args[1]->ToObject())->GetValue();
// end convert_from_v8 block
  baton->branch = from_branch;
// start convert_from_v8 block
  const git_annotated_commit * from_upstream;
  from_upstream = ObjectWrap::Unwrap<GitAnnotatedCommit>(args[2]->ToObject())->GetValue();
// end convert_from_v8 block
  baton->upstream = from_upstream;
// start convert_from_v8 block
  const git_annotated_commit * from_onto;
  from_onto = ObjectWrap::Unwrap<GitAnnotatedCommit>(args[3]->ToObject())->GetValue();
// end convert_from_v8 block
  baton->onto = from_onto;
// start convert_from_v8 block
  const git_signature * from_signature;
  from_signature = ObjectWrap::Unwrap<GitSignature>(args[4]->ToObject())->GetValue();
// end convert_from_v8 block
  baton->signature = from_signature;
// start convert_from_v8 block
  const git_rebase_options * from_opts;
  from_opts = ObjectWrap::Unwrap<GitRebaseOptions>(args[5]->ToObject())->GetValue();
// end convert_from_v8 block
  baton->opts = from_opts;

  NanCallback *callback = new NanCallback(Local<Function>::Cast(args[6]));
  InitWorker *worker = new InitWorker(baton, callback);
  if (!args[0]->IsUndefined() && !args[0]->IsNull())
    worker->SaveToPersistent("repo", args[0]->ToObject());
  if (!args[1]->IsUndefined() && !args[1]->IsNull())
    worker->SaveToPersistent("branch", args[1]->ToObject());
  if (!args[2]->IsUndefined() && !args[2]->IsNull())
    worker->SaveToPersistent("upstream", args[2]->ToObject());
  if (!args[3]->IsUndefined() && !args[3]->IsNull())
    worker->SaveToPersistent("onto", args[3]->ToObject());
  if (!args[4]->IsUndefined() && !args[4]->IsNull())
    worker->SaveToPersistent("signature", args[4]->ToObject());
  if (!args[5]->IsUndefined() && !args[5]->IsNull())
    worker->SaveToPersistent("opts", args[5]->ToObject());

  NanAsyncQueueWorker(worker);
  NanReturnUndefined();
}

void GitRebase::InitWorker::Execute()
{
  int result = git_rebase_init(
                 &baton->out,baton->repo,baton->branch,baton->upstream,baton->onto,baton->signature,baton->opts    );

  baton->error_code = result;

  if (result != GIT_OK && giterr_last() != NULL) {
    baton->error = git_error_dup(giterr_last());
  }

}

void GitRebase::InitWorker::HandleOKCallback()
{
  TryCatch try_catch;

  if (baton->error_code == GIT_OK) {
    Handle<v8::Value> to;
// start convert_to_v8 block

    if (baton->out != NULL) {
      // GitRebase baton->out
      to = GitRebase::New((void *)baton->out, false);
    } else {
      to = NanNull();
    }

// end convert_to_v8 block
    Handle<v8::Value> result = to;
    Handle<v8::Value> argv[2] = {
      NanNull(),
      result
    };
    callback->Call(2, argv);
  } else {
    if (baton->error) {
      Handle<v8::Value> argv[1] = {
        NanError(baton->error->message)
      };
      callback->Call(1, argv);
      if (baton->error->message)
        free((void *)baton->error->message);
      free((void *)baton->error);
    } else {
      callback->Call(0, NULL);
    }

  }

  if (try_catch.HasCaught()) {
    node::FatalException(try_catch);
  }


  delete baton;
}


/*
 * @param RebaseOptions opts
   * @param Number version
     * @return Number  result    */
NAN_METHOD(GitRebase::InitOptions)
{
  NanEscapableScope();

  if (args.Length() == 0 || !args[0]->IsObject()) {
    return NanThrowError("RebaseOptions opts is required.");
  }

  if (args.Length() == 1 || !args[1]->IsNumber()) {
    return NanThrowError("Number version is required.");
  }

// start convert_from_v8 block
  git_rebase_options * from_opts;
  from_opts = ObjectWrap::Unwrap<GitRebaseOptions>(args[0]->ToObject())->GetValue();
// end convert_from_v8 block
// start convert_from_v8 block
  unsigned int from_version;
  from_version = (unsigned int)   args[1]->ToNumber()->Value();
// end convert_from_v8 block
  int result = git_rebase_init_options(
                 from_opts
                 ,from_version
               );


  Handle<v8::Value> to;
// start convert_to_v8 block
  to = NanNew<Number>( result);
  // end convert_to_v8 block
  NodeGitPsueodoNanReturnEscapingValue(to);
}

/*
  * @param Repository repo
    * @param Rebase callback
   */
NAN_METHOD(GitRebase::Open)
{
  NanScope();

  if (args.Length() == 0 || !args[0]->IsObject()) {
    return NanThrowError("Repository repo is required.");
  }

  if (args.Length() == 1 || !args[1]->IsFunction()) {
    return NanThrowError("Callback is required and must be a Function.");
  }

  OpenBaton* baton = new OpenBaton;

  baton->error_code = GIT_OK;
  baton->error = NULL;

// start convert_from_v8 block
  git_repository * from_repo;
  from_repo = ObjectWrap::Unwrap<GitRepository>(args[0]->ToObject())->GetValue();
// end convert_from_v8 block
  baton->repo = from_repo;

  NanCallback *callback = new NanCallback(Local<Function>::Cast(args[1]));
  OpenWorker *worker = new OpenWorker(baton, callback);
  if (!args[0]->IsUndefined() && !args[0]->IsNull())
    worker->SaveToPersistent("repo", args[0]->ToObject());

  NanAsyncQueueWorker(worker);
  NanReturnUndefined();
}

void GitRebase::OpenWorker::Execute()
{
  int result = git_rebase_open(
                 &baton->out,baton->repo    );

  baton->error_code = result;

  if (result != GIT_OK && giterr_last() != NULL) {
    baton->error = git_error_dup(giterr_last());
  }

}

void GitRebase::OpenWorker::HandleOKCallback()
{
  TryCatch try_catch;

  if (baton->error_code == GIT_OK) {
    Handle<v8::Value> to;
// start convert_to_v8 block

    if (baton->out != NULL) {
      // GitRebase baton->out
      to = GitRebase::New((void *)baton->out, false);
    } else {
      to = NanNull();
    }

// end convert_to_v8 block
    Handle<v8::Value> result = to;
    Handle<v8::Value> argv[2] = {
      NanNull(),
      result
    };
    callback->Call(2, argv);
  } else {
    if (baton->error) {
      Handle<v8::Value> argv[1] = {
        NanError(baton->error->message)
      };
      callback->Call(1, argv);
      if (baton->error->message)
        free((void *)baton->error->message);
      free((void *)baton->error);
    } else {
      callback->Call(0, NULL);
    }

  }

  if (try_catch.HasCaught()) {
    node::FatalException(try_catch);
  }


  delete baton;
}


/*
   * @param Number idx
     * @return RebaseOperation  result    */
NAN_METHOD(GitRebase::OperationByindex)
{
  NanEscapableScope();

  if (args.Length() == 0 || !args[0]->IsNumber()) {
    return NanThrowError("Number idx is required.");
  }

// start convert_from_v8 block
  size_t from_idx;
  from_idx = (size_t)   args[0]->ToNumber()->Value();
// end convert_from_v8 block
  git_rebase_operation * result = git_rebase_operation_byindex(
                                    ObjectWrap::Unwrap<GitRebase>(args.This())->GetValue()
                                    ,from_idx
                                  );

  // null checks on pointers
  if (!result) {
    NodeGitPsueodoNanReturnEscapingValue(NanUndefined());
  }

  Handle<v8::Value> to;
// start convert_to_v8 block

  if (result != NULL) {
    // GitRebaseOperation result
    to = GitRebaseOperation::New((void *)result, false);
  } else {
    to = NanNull();
  }

// end convert_to_v8 block
  NodeGitPsueodoNanReturnEscapingValue(to);
}

/*
     * @return Number  result    */
NAN_METHOD(GitRebase::OperationCurrent)
{
  NanEscapableScope();

  size_t result = git_rebase_operation_current(
                    ObjectWrap::Unwrap<GitRebase>(args.This())->GetValue()
                  );


  Handle<v8::Value> to;
// start convert_to_v8 block
  to = NanNew<Number>( result);
  // end convert_to_v8 block
  NodeGitPsueodoNanReturnEscapingValue(to);
}

/*
     * @return Number  result    */
NAN_METHOD(GitRebase::OperationEntrycount)
{
  NanEscapableScope();

  size_t result = git_rebase_operation_entrycount(
                    ObjectWrap::Unwrap<GitRebase>(args.This())->GetValue()
                  );


  Handle<v8::Value> to;
// start convert_to_v8 block
  to = NanNew<Number>( result);
  // end convert_to_v8 block
  NodeGitPsueodoNanReturnEscapingValue(to);
}
Persistent<Function> GitRebase::constructor_template;
